
# This code is a part of Slash, and is released under the GPL.
# Copyright 1997-2001 by Open Source Development Network. See README
# and COPYING for more information, or see http://slashcode.com/.

# This file has seen modifications to turn this file, which was at one point a Perl CGI script into a Apache mod_perl2 module handler. This file and code in it is derived from works of the original authors and copyright owners (Open Source Developers Network). The software package "Slashcode" was released as GPL. This gives others the legal right to use, modify, and redistribute modified versions of Slashcode.

# Code was added to the GPL'ed version of what OSDN has released at version 2.2.6. Namely, code was added so that users client browsers can authenticated using passwords and athentication data stored in HTTP cookies. These modifications are copyright 2014 by Troye "taj" Johnson <troyejohnson@yandex.com>. The license for these modifications is GPLv2, or later.

package slash_search_pl; # for /search.pl

use strict;

use Apache2::Const qw(REDIRECT OK);

use Slash;
use Slash::Display;
use Slash::Utility;
use Slash::Search;
use Slash::XML;

# check if $ENV{SLASH_VIRTUAL_USER} and $ENV{SLASH_CONFIG} defined in httpd.conf
slash_preinit_check();

# comes from httpd.conf
my $VIRTUAL_USER = $ENV{SLASH_VIRTUAL_USER};

# must be called before handler() is run
slash_api_init($VIRTUAL_USER);


=item slash_index_pl::handler

Resides in: slash_index_pl.pm

This handler is for the /index.pl URL. Since this is a modperl handler, the '<Location></Location>' tags will have to set the URL, so that Apache will map the URL to the proper module handler in order for the request to be served.

Most of these params will not be handled by the handler() or interpret their meaning. main() will handle that stuff.

All handler does is some pre-processing to set-up the main SLASH_STRUCTS (you can fetch these with getCurrentUser, getCurrentForm, and of course getCurrentDB, most or all of the getCurrent*() subs are used to fetch so call "SLASH_STRUCTS". They are just HASH references used by the API.) so that the handler() can call main(), and thus the module handler has done it's job.

Argument list for this handler. "op" seems like the /index.pl links back to itself so that the site can re-display the page. "section" seems to also refer back to the site to display things.

USER_STRUCT; needed

=cut

sub handler {
    my ( $r ) = @_;

    die "slash_users_pl: slash_api_init() not called"
	unless SLASH_API_INIT_OK;

    my $uid_Param = undef; # This available after authentication process

    my $form_Param = slash_get_params( $r );
    my $cookie_args_Param = slash_get_cookies( $r );
    my $uri_Param = $r->uri(); # e.g, /index.html, /index.pl, or /foo.(html|pl)

    my $method_Param = $r->the_request(); $method_Param =~ s/^(\S*)/$1/;
    $method_Param = $1; # should now be either GET, POST

    my ($unickname, $upasswd); # params from form
    $unickname = $form_Param->{unickname} if $form_Param->{unickname};
    $upasswd = $form_Param->{upasswd} if $form_Param->{upasswd};

    #
    # Attempt to authenticate user
    #

    # $auth_type: 1 is for form login. 2 is for authentication via cookie. Set this to "2" as /users.pl is the only script which can auth w/ "auth_type = 1" -- which is for username and password logins only. This script can authenticate only using cookies.
    my $auth_type = 2;
 
    my $authing_user = undef; # uid of authenticated user, else AnonymousCoward
    my $auth_success = 0; # 1 for successful auth, 0 for authentication failure
    my $auth_uid_field = 0; # First element in array returned by Slash::DB::gutUserAuthenticate()
      
    # For cookie authentication: authenticate with cookie
    if ($auth_type == 2) {
	( $unickname, $upasswd ) = eatUserCookie($cookie_args_Param->{users});

	$authing_user = slash_web_auth( $unickname, $upasswd, 2);
	$auth_success = 1 if $authing_user;

	# `logger 'handler: auth type => 2'`; 
	# `logger 'handler: users cookie in val => '$cookie_args->{users}''`;
	# `logger 'handler: eatUserCookie: n => $unickname, p => $upasswd'`; 
    }
    
    
    if ($auth_success == 1) {
	$uid_Param = $authing_user->[$auth_uid_field];    
    }

    # ** DISABLED: this code designed to follow up after a "auth_type = 1" login
    #
    # if ( ($auth_success == 1) && ($auth_type == 1) ) {
    # 	# ** Set cookie if auth_type is login
	
    # 	# Does not matter whether $upasswd is encrypted or not
    # 	my $users_cookie_data
    # 	    = bakeUserCookie( $authing_user->[$auth_uid_field], $upasswd);
    # 	setCookie('users', $users_cookie_data);
	
    # 	# `logger 'users cookie out value => '$users_cookie_data''`;
    # 	# `logger 'handler: auth was good'`;
    # }
    

    # If User did not authenticate, then they will be anonymous
    if (! defined $uid_Param) {
	$uid_Param = SLASH_ANON_COWARD_UID();
	# `logger 'handler: auth not successful'`; 
    }

    createEnvironment($VIRTUAL_USER, $form_Param, $uid_Param, $uri_Param, $cookie_args_Param, $method_Param);
    main();
    return Apache2::Const::OK;
}


# ** main() does most of the work

#################################################################
sub main {
    my %ops = (
        comments => \&commentSearch,
        users    => \&userSearch,
        stories  => \&storySearch
    );
    my %ops_rss = (
        comments => \&commentSearchRSS,
        users    => \&userSearchRSS,
        stories  => \&storySearchRSS
    );

    my $constants = getCurrentStatic();
    my $form      = getCurrentForm();

    my ( $slashdb, $searchDB );

    if ( $constants->{search_db_user} ) {
        $slashdb  = getObject( 'Slash::DB',     $constants->{search_db_user} );
        $searchDB = getObject( 'Slash::Search', $constants->{search_db_user} );
    }
    else {
        $slashdb  = getCurrentDB();
        $searchDB = Slash::Search->new( getCurrentVirtualUser() );
    }

    # Set some defaults
    $form->{query}   ||= '';
    $form->{section} ||= '';
    $form->{section} = ''
      if $form->{section} eq 'index';   # XXX fix this right, do a {realsection}
    $form->{threshold} ||= getCurrentUser('threshold');

    # get rid of bad characters
    $form->{query} =~ s/[^A-Z0-9'. ]/ /gi;

    # truncate query length
    if ( length( $form->{query} ) > 40 ) {
        $form->{query} = substr( $form->{query}, 0, 40 );
    }

    # The default search operation is to search stories.
    $form->{op} ||= 'stories';

    if ( $form->{content_type} eq 'rss' ) {

        # Here, panic mode is handled within the individual funcs.
        # We want to return valid (though empty) RSS data even
        # when search is down.
        $form->{op} = 'stories' if !exists( $ops_rss{ $form->{op} } );
        $ops_rss{ $form->{op} }->( $form, $constants, $slashdb, $searchDB );
    }
    else {
        header( "$constants->{sitename}: Search $form->{query}",
            $form->{section} );
        titlebar( "99%", "Searching $form->{query}" );
        $form->{op} = 'stories' if !exists( $ops{ $form->{op} } );

        # Here, panic mode is handled without needing to call the
        # individual search subroutines;  we're going to tell the
        # user the same thing in each case anyway.
        if ( $constants->{panic} >= 1 or $constants->{search_google} ) {
            slashDisplay('nosearch');
        }
        else {
            slashDisplay(
                'searchform',
                {
                    sections => _sections(),
                    topics   => _topics(),
                    tref     => $slashdb->getTopic( $form->{topic} ),
                    op       => $form->{op},
                    authors  => _authors(),
                    'sort'   => _sort(),
                }
            );
            if ( $ops{ $form->{op} } ) {
                $ops{ $form->{op} }->( $form, $constants, $slashdb, $searchDB );
            }
        }

        footer();
    }

    writeLog( $form->{query} )
      if $form->{op} =~ /^(?:comments|stories|users)$/;
}

#################################################################
# Ugly isn't it?
sub _authors {
    my $slashdb = getCurrentDB();
    my $authors = $slashdb->getDescriptions('all-authors');
    $authors->{''} = 'All Authors';

    return $authors;
}

#################################################################
# Ugly isn't it?
sub _topics {
    my $slashdb = getCurrentDB();
    my $topics  = $slashdb->getDescriptions('topics');
    $topics->{''} = 'All Topics';

    return $topics;
}

#################################################################
sub _sort {

    # this needs to be in the database -- pudge
    my $sort;
    $sort->{''}      = 'Default Order';
    $sort->{'date'}  = 'Order By Date';
    $sort->{'score'} = 'Order By Score';

    return $sort;
}

#################################################################
# Ugly isn't it?
sub _sections {
    my $slashdb  = getCurrentDB();
    my $sections = $slashdb->getDescriptions('sections');
    $sections->{''} = 'All Sections';

    return $sections;
}

#################################################################
sub _buildargs {
    my ($form) = @_;
    my $uri;

    for (qw[threshold query author op topic section]) {
        my $x = "";
        $x = $form->{$_} if defined $form->{$_} && $x eq "";
        $x =~ s/ /+/g;
        $uri .= "$_=$x&" unless $x eq "";
    }
    $uri =~ s/&$//;

    return fixurl($uri);
}

#################################################################
sub commentSearch {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $comments =
      $searchDB->findComments( $form, $start,
        $constants->{search_default_display} + 1,
        $form->{sort} );

    # check for extra articles ... we request one more than we need
    # and if we get the extra one, we know we have extra ones, and
    # we pop it off
    my $forward;
    if ( @$comments == $constants->{search_default_display} + 1 ) {
        pop @$comments;
        $forward = $start + $constants->{search_default_display};
    }
    else {
        $forward = 0;
    }

    # if there are less than search_default_display remaning,
    # just set it to 0
    my $back;
    if ( $start > 0 ) {
        $back = $start - $constants->{search_default_display};
        $back = $back > 0 ? $back : 0;
    }
    else {
        $back = -1;
    }

    slashDisplay(
        'commentsearch',
        {
            comments => $comments,
            back     => $back,
            forward  => $forward,
            args     => _buildargs($form),
            start    => $start,
        }
    );
}

#################################################################
sub userSearch {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $users =
      $searchDB->findUsers( $form, $start,
        $constants->{search_default_display} + 1,
        $form->{sort} );

    # check for extra articles ... we request one more than we need
    # and if we get the extra one, we know we have extra ones, and
    # we pop it off
    my $forward;
    if ( @$users == $constants->{search_default_display} + 1 ) {
        pop @$users;
        $forward = $start + $constants->{search_default_display};
    }
    else {
        $forward = 0;
    }

    # if there are less than search_default_display remaning,
    # just set it to 0
    my $back;
    if ( $start > 0 ) {
        $back = $start - $constants->{search_default_display};
        $back = $back > 0 ? $back : 0;
    }
    else {
        $back = -1;
    }

    slashDisplay(
        'usersearch',
        {
            users   => $users,
            back    => $back,
            forward => $forward,
            args    => _buildargs($form),
        }
    );
}

#################################################################
sub storySearch {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $stories =
      $searchDB->findStory( $form, $start,
        $constants->{search_default_display} + 1,
        $form->{sort} );

    # check for extra articles ... we request one more than we need
    # and if we get the extra one, we know we have extra ones, and
    # we pop it off
    my $forward;
    if ( @$stories == $constants->{search_default_display} + 1 ) {
        pop @$stories;
        $forward = $start + $constants->{search_default_display};
    }
    else {
        $forward = 0;
    }

    # if there are less than search_default_display remaning,
    # just set it to 0
    my $back;
    if ( $start > 0 ) {
        $back = $start - $constants->{search_default_display};
        $back = $back > 0 ? $back : 0;
    }
    else {
        $back = -1;
    }

    slashDisplay(
        'storysearch',
        {
            stories => $stories,
            back    => $back,
            forward => $forward,
            args    => _buildargs($form),
            start   => $start,
        }
    );
}

#################################################################
sub commentSearchRSS {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $comments;
    if ( $constants->{panic} >= 1 or $constants->{search_google} ) {
        $comments = [];
    }
    else {
        $comments = $searchDB->findComments( $form, $start, 15, $form->{sort} );
    }

    my @items;
    for my $entry (@$comments) {
        my $time = timeCalc( $entry->[3] );
        push @items,
          {
            title  => "$entry->[5] ($time)",
            'link' => (
                $constants->{absolutedir}
                  . "/comments.pl?sid=entry->[1]&pid=entry->[4]#entry->[10]"
            ),
          };
    }

    xmlDisplay(
        rss => {
            channel => {
                title       => "$constants->{sitename} Comment Search",
                'link'      => "$constants->{absolutedir}/search.pl",
                description => "$constants->{sitename} Comment Search",
            },
            image => 1,
            items => \@items
        }
    );
}

#################################################################
sub userSearchRSS {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $users;
    if ( $constants->{panic} >= 1 or $constants->{search_google} ) {
        $users = [];
    }
    else {
        $users = $searchDB->findUsers( $form, $start, 15, $form->{sort} );
    }

    my @items;
    for my $entry (@$users) {
        my $time = timeCalc( $entry->[3] );
        push @items,
          {
            title => "$entry->[1]",
            'link' =>
              ( $constants->{absolutedir} . '/users.pl?nick=' . $entry->[1] ),
          };
    }

    xmlDisplay(
        rss => {
            channel => {
                title       => "$constants->{sitename} User Search",
                'link'      => "$constants->{absolutedir}/search.pl",
                description => "$constants->{sitename} User Search",
            },
            image => 1,
            items => \@items
        }
    );
}

#################################################################
sub storySearchRSS {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $stories;
    if ( $constants->{panic} >= 1 or $constants->{search_google} ) {
        $stories = [];
    }
    else {
        $stories = $searchDB->findStory( $form, $start, 15, $form->{sort} );
    }

    my @items;
    for my $entry (@$stories) {
        my $time = timeCalc( $entry->[3] );
        push @items,
          {
            title => "$entry->[1] ($time)",
            'link' =>
              ( $constants->{absolutedir} . '/article.pl?sid=' . $entry->[2] ),
          };
    }

    xmlDisplay(
        rss => {
            channel => {
                title       => "$constants->{sitename} Story Search",
                'link'      => "$constants->{absolutedir}/search.pl",
                description => "$constants->{sitename} Story Search",
            },
            image => 1,
            items => \@items
        }
    );
}

#################################################################
# Do not enable -Brian
sub findRetrieveSite {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $feeds =
      $searchDB->findRetrieveSite( $form->{query}, $start,
        $constants->{search_default_display} + 1,
        $form->{sort} );

    # check for extra feeds ... we request one more than we need
    # and if we get the extra one, we know we have extra ones, and
    # we pop it off
    my $forward;
    if ( @$feeds == $constants->{search_default_display} + 1 ) {
        pop @$feeds;
        $forward = $start + $constants->{search_default_display};
    }
    else {
        $forward = 0;
    }

    # if there are less than search_default_display remaning,
    # just set it to 0
    my $back;
    if ( $start > 0 ) {
        $back = $start - $constants->{search_default_display};
        $back = $back > 0 ? $back : 0;
    }
    else {
        $back = -1;
    }

    slashDisplay(
        'retrievedsites',
        {
            feeds   => $feeds,
            back    => $back,
            forward => $forward,
            start   => $start,
        }
    );
}

#################################################################
# Do not enable -Brian
sub findRetrieveSiteRSS {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $feeds =
      $searchDB->findFeeds( $form->{query}, $start, 15, $form->{sort} );

    # I am aware that the link has to be improved.
    my @items;
    for my $entry (@$feeds) {
        my $time = timeCalc( $entry->[8] );
        push @items,
          {
            title  => "$entry->[2] ($time)",
            'link' => (
                $constants->{absolutedir}
                  . "/users.pl?op=preview&bid=entry->[0] %]"
            ),
          };
    }

    xmlDisplay(
        rss => {
            channel => {
                title       => "$constants->{sitename} Retrieve Site Search",
                'link'      => "$constants->{absolutedir}/search.pl",
                description => "$constants->{sitename} Retrieve Site Search",
            },
            image => 1,
            items => \@items
        }
    );
}

#################################################################
# Do not enable -Brian
sub findJournalEntry {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $entries =
      $searchDB->findJournalEntry( $form, $start,
        $constants->{search_default_display} + 1,
        $form->{sort} );

    # check for extra articles ... we request one more than we need
    # and if we get the extra one, we know we have extra ones, and
    # we pop it off
    my $forward;
    if ( @$entries == $constants->{search_default_display} + 1 ) {
        pop @$entries;
        $forward = $start + $constants->{search_default_display};
    }
    else {
        $forward = 0;
    }

    # if there are less than search_default_display remaning,
    # just set it to 0
    my $back;
    if ( $start > 0 ) {
        $back = $start - $constants->{search_default_display};
        $back = $back > 0 ? $back : 0;
    }
    else {
        $back = -1;
    }

    slashDisplay(
        'storysearch',
        {
            entries => $entries,
            back    => $back,
            forward => $forward,
            args    => _buildargs($form),
            start   => $start,
        }
    );
}

#################################################################
# Do not enable -Brian
# do not WRITE in the first place -- pudge
# Writing is fine if it is not enabled --Brian
sub findJournalEntryRSS {
    my ( $form, $constants, $slashdb, $searchDB ) = @_;

    my $start = $form->{start} || 0;
    my $entries =
      $searchDB->findJournalEntry( $form, $start, 15, $form->{sort} );

    my @items;
    for my $entry (@$entries) {
        my $time = timeCalc( $entry->[3] );
        push @items,
          {
            title => "$entry->[1] ($time)",
            'link' =>
              ( $constants->{absolutedir} . '/article.pl?sid=' . $entry->[2] ),
          };
    }

    xmlDisplay(
        rss => {
            channel => {
                title       => "$constants->{sitename} Journal Search",
                'link'      => "$constants->{absolutedir}/search.pl",
                description => "$constants->{sitename} Journal Search",
            },
            image => 1,
            items => \@items
        }
    );
}


1;
