# This code is a part of Slash, and is released under the GPL.
# Copyright 1997-2001 by Open Source Development Network. See README
# and COPYING for more information, or see http://slashcode.com/.

# This file has seen modifications to turn this file, which was at one point a Perl CGI script into a Apache mod_perl2 module handler. This file and code in it is derived from works of the original authors and copyright owners (Open Source Developers Network). The software package "Slashcode" was released as GPL. This gives others the legal right to use, modify, and redistribute modified versions of Slashcode.

# Code was added to the GPL'ed version of what OSDN has released at version 2.2.6. Namely, code was added so that users client browsers can authenticated using passwords and athentication data stored in HTTP cookies. These modifications are copyright 2014 by Troye "taj" Johnson <troyejohnson@yandex.com>. The license for these modifications is GPLv2, or later.

package slash_404_pl; # for /404.pl

use strict;

use Slash;
use Slash::Display;
use Slash::Utility;
use Apache2::Const qw(OK);

# check if $ENV{SLASH_VIRTUAL_USER} and $ENV{SLASH_CONFIG} defined in httpd.conf
slash_preinit_check();

# comes from httpd.conf
my $VIRTUAL_USER = $ENV{SLASH_VIRTUAL_USER};

# must be called before handler() is run
slash_api_init($VIRTUAL_USER);


=item slash_404_pl::handler

Resides in: slash_404_pl.pm

This handler is for the /404.pl URL. Since this is a modperl handler, the '<Location></Location>' tags will have to set the URL, so that Apache will map the URL to the proper module handler in order for the request to be served.

Most of these params will not be handled by the handler() or interpret their meaning. main() will handle that stuff.

All handler does is some pre-processing to set-up the main SLASH_STRUCTS so that the handler() can call main(), and thus the module handler has done it's job.

=cut

sub handler {
    my ( $r ) = @_;

    die "slash_404_pl: slash_api_init() not called"
	unless SLASH_API_INIT_OK;

    my $form_Param = slash_get_params( $r );

    # We modify $ENV{REQUEST_URI} because main() references it
    $ENV{REQUEST_URI} = $r->unparsed_uri();
  
    createCurrentForm(fixParam(%$form_Param)); # main() needs form data

    # ** Disabled createEnvironment(). No use for user_struct information:
    #
    # createEnvironment($VIRTUAL_USER, $form_Param, $uid_Param, $uri_Param, $cookie_args_Param, $method_Param);
    main();
    return Apache2::Const::OK;
}



sub main {
    my $constants = getCurrentStatic();
    my $form      = getCurrentForm();
    $ENV{REQUEST_URI} ||= '';

    # catch old .shtml links ... need to check for other schemes, too?
    if ( $ENV{REQUEST_URI} =~ m|^/?\w+/(\d\d/\d\d/\d\d/\d+)\.shtml$| ) {
        redirect("$constants->{rootdir}/article.pl?sid=$1");
        return;
    }

    my $url = strip_literal( substr( $ENV{REQUEST_URI}, 1 ) );
    my $admin = $constants->{adminmail};

    header( '404 File Not Found', $form->{section} );

    my ( $new_url, $errnum ) = fixHref( $ENV{REQUEST_URI}, 1 );

    if ( $errnum && $errnum !~ /^\d+$/ ) {
        slashDisplay(
            'main',
            {
                url     => $new_url,
                origin  => $url,
                message => $errnum,
            }
        );
    }
    else {
        slashDisplay(
            'main',
            {
                error  => $errnum,
                url    => $new_url,
                origin => $url,
            }
        );
    }

    writeLog($url);
    footer();
}



1;
