# This code is a part of Slash, and is released under the GPL.
# Copyright 1997-2001 by Open Source Development Network. See README
# and COPYING for more information, or see http://slashcode.com/.

# This file has seen modifications to turn this file, which was at one point a Perl CGI script into a Apache mod_perl2 module handler. This file and code in it is derived from works of the original authors and copyright owners (Open Source Developers Network). The software package "Slashcode" was released as GPL. This gives others the legal right to use, modify, and redistribute modified versions of Slashcode.

# Code was added to the GPL'ed version of what OSDN has released at version 2.2.6. Namely, code was added so that users client browsers can authenticated using passwords and athentication data stored in HTTP cookies. These modifications are copyright 2014 by Troye "taj" Johnson <troyejohnson@yandex.com>. The license for these modifications is GPLv2, or later.

package slash_pollBooth_pl; # for /pollBooth.pl

use strict;

use Slash;
use Slash::Display;
use Slash::Utility;

use Apache2::Const qw(REDIRECT OK);

# check if $ENV{SLASH_VIRTUAL_USER} and $ENV{SLASH_CONFIG} defined in httpd.conf
slash_preinit_check();

# comes from httpd.conf
my $VIRTUAL_USER = $ENV{SLASH_VIRTUAL_USER};

# must be called before handler() is run
slash_api_init($VIRTUAL_USER);


=item slash_index_pl::handler

Resides in: slash_index_pl.pm

This handler is for the /index.pl URL. Since this is a modperl handler, the '<Location></Location>' tags will have to set the URL, so that Apache will map the URL to the proper module handler in order for the request to be served.

Most of these params will not be handled by the handler() or interpret their meaning. main() will handle that stuff.

All handler does is some pre-processing to set-up the main SLASH_STRUCTS so that the handler() can call main(), and thus the module handler has done it's job.

USER_STRUCT; yes needs it

=cut

sub handler {
    my ( $r ) = @_;

    die "slash_users_pl: slash_api_init() not called"
	unless SLASH_API_INIT_OK;

    my $uid_Param = undef; # This available after authentication process

    my $form_Param = slash_get_params( $r );
    my $cookie_args_Param = slash_get_cookies( $r );
    my $uri_Param = $r->uri(); # e.g, /index.html, /index.pl, or /foo.(html|pl)

    my $method_Param = $r->the_request(); $method_Param =~ s/^(\S*)/$1/;
    $method_Param = $1; # should now be either GET, POST

    my ($unickname, $upasswd); # params from form
    $unickname = $form_Param->{unickname} if $form_Param->{unickname};
    $upasswd = $form_Param->{upasswd} if $form_Param->{upasswd};

    #
    # Attempt to authenticate user
    #

    # $auth_type: 1 is for form login. 2 is for authentication via cookie. Set this to "2" as /users.pl is the only script which can auth w/ "auth_type = 1" -- which is for username and password logins only. This script can authenticate only using cookies.
    my $auth_type = 2;
 
    my $authing_user = undef; # uid of authenticated user, else AnonymousCoward
    my $auth_success = 0; # 1 for successful auth, 0 for authentication failure
    my $auth_uid_field = 0; # First element in array returned by Slash::DB::gutUserAuthenticate()
      
    # For cookie authentication: authenticate with cookie
    if ($auth_type == 2) {
	( $unickname, $upasswd ) = eatUserCookie($cookie_args_Param->{users});

	$authing_user = slash_web_auth( $unickname, $upasswd, 2);
	$auth_success = 1 if $authing_user;

	# `logger 'handler: auth type => 2'`; 
	# `logger 'handler: users cookie in val => '$cookie_args->{users}''`;
	# `logger 'handler: eatUserCookie: n => $unickname, p => $upasswd'`; 
    }
    
    
    if ($auth_success == 1) {
	$uid_Param = $authing_user->[$auth_uid_field];    
    }

    # ** DISABLED: this code designed to follow up after a "auth_type = 1" login
    #
    # if ( ($auth_success == 1) && ($auth_type == 1) ) {
    # 	# ** Set cookie if auth_type is login
	
    # 	# Does not matter whether $upasswd is encrypted or not
    # 	my $users_cookie_data
    # 	    = bakeUserCookie( $authing_user->[$auth_uid_field], $upasswd);
    # 	setCookie('users', $users_cookie_data);
	
    # 	# `logger 'users cookie out value => '$users_cookie_data''`;
    # 	# `logger 'handler: auth was good'`;
    # }
    

    # If User did not authenticate, then they will be anonymous
    if (! defined $uid_Param) {
	$uid_Param = SLASH_ANON_COWARD_UID();
	# `logger 'handler: auth not successful'`; 
    }

    createEnvironment($VIRTUAL_USER, $form_Param, $uid_Param, $uri_Param, $cookie_args_Param, $method_Param);
    main();
    return Apache2::Const::OK;
}


#################################################################
sub main {
    my $slashdb = getCurrentDB();
    my $form    = getCurrentForm();

    my %ops = (
        edit     => \&editpoll,
        save     => \&savepoll,
        'delete' => \&deletepolls,
        list     => \&listpolls,
        default  => \&default,
        vote     => \&vote,
        get      => \&poll_booth,
    );

    my $op = $form->{op};
    if ( defined $form->{'aid'} && $form->{'aid'} !~ /^\-?\d$/ ) {
        undef $form->{'aid'};
    }

    header( getData('title'), $form->{section} );

    $op = 'default' unless $ops{ $form->{op} };
    $ops{$op}->($form);

    writeLog( $form->{'qid'} );
    footer();
}

#################################################################
sub poll_booth {
    my ($form) = @_;

    print pollbooth( $form->{'qid'}, 0, 1 );
}

#################################################################
sub default {
    my ($form) = @_;

    if ( !$form->{'qid'} ) {
        listpolls(@_);
    }
    elsif ( !defined $form->{'aid'} ) {
        poll_booth(@_);
    }
    else {
        my $vote = vote(@_);
        if ( getCurrentStatic('poll_discussions') ) {
            my $slashdb = getCurrentDB();
            my $discussion_id =
              $slashdb->getPollQuestion( $form->{'qid'}, 'discussion' );
            my $discussion = $slashdb->getDiscussion($discussion_id);
            printComments($discussion) if $discussion;
        }
    }
}

#################################################################
sub editpoll {
    my ($form) = @_;

    my ($qid) = $form->{'qid'};
    unless ( getCurrentUser('is_admin') ) {
        default(@_);
        return;
    }
    my $slashdb = getCurrentDB();

    my $currentqid = $slashdb->getVar( 'currentqid', 'value' )
      if $qid;
    my $question =
      $slashdb->getPollQuestion( $qid, [ 'question', 'voters', 'sid' ] )
      if $qid;

    my $answers;
    if ($question) {
        $question->{voters} ||= 0;
        $answers = $slashdb->getPollAnswers( $qid, [ 'answer', 'votes' ] )
          if $qid;
    }
    else {
        $question->{voters} ||= 0;
        $question->{question} = $form->{question};
        $question->{sid}      = $form->{sid};
    }

    slashDisplay(
        'editpoll',
        {
            title => getData( 'edit_poll_title', { qid => $qid } ),
            checked => $currentqid eq $qid ? ' CHECKED' : '',
            qid => $qid,
            question => $question,
            answers  => $answers,
        }
    );
}

#################################################################
sub savepoll {
    my ($form) = @_;

    unless ( getCurrentUser('is_admin') ) {
        default(@_);
        return;
    }
    my $slashdb   = getCurrentDB();
    my $constants = getCurrentStatic();
    slashDisplay('savepoll');

    #We are lazy, we just pass along $form as a $poll
    my $qid = $slashdb->savePollQuestion($form);

    # we have a problem here.  if you attach the poll to an SID,
    # and then unattach it, it will still be attached to that SID
    # until you either change it manually in the DB, or attach it
    # to a new SID.  Deal with it, or send in a patch.  The logic
    # to deal with it otherwise is too complex to be warranted
    # given the infrequency of the circumstance. -- pudge

    if ( $constants->{poll_discussions} ) {
        my $poll = $slashdb->getPollQuestion($qid);
        my $discussion;
        if ( $poll->{sid} ) {

            # if sid lookup fails, then $discussion is empty,
            # and the poll's discussion is not set
            $discussion = $slashdb->getStory( $form->{sid}, 'discussion' );
        }
        elsif ( !$poll->{discussion} ) {
            $discussion = $slashdb->createDiscussion(
                {
                    title => $form->{question},
                    topic => $form->{topic},
                    url =>
                      "$constants->{rootdir}/pollBooth.pl?op=vote&qid=$qid",
                }
            );
        }

        # if it already has a discussion (so $discussion is not set),
        # or discussion ID is unchanged, don't bother setting
        $slashdb->setPollQuestion( $qid, { discussion => $discussion } )
          if $discussion && $discussion != $poll->{discussion};
    }
    $slashdb->setStory( $form->{sid}, { qid => $qid } ) if $form->{sid};
}

#################################################################
sub vote {
    my ($form) = @_;

    my $qid = $form->{'qid'};
    my $aid = $form->{'aid'};

    return unless $qid;

    my $slashdb = getCurrentDB();

    my (%all_aid) =
      map { ( $_->[0], 1 ) } @{ $slashdb->getPollAnswers( $qid, ['aid'] ) };

    if ( !keys %all_aid ) {
        print getData('invalid');

        # Non-zero denotes error condition and that comments
        # should not be printed.
        return;
    }

    my $question = $slashdb->getPollQuestion( $qid, [ 'voters', 'question' ] );
    my $notes = getData('display');
    if ( getCurrentUser('is_anon') and !getCurrentStatic('allow_anonymous') ) {
        $notes = getData('anon');
    }
    elsif ( $aid > 0 ) {
        my $id = $slashdb->getPollVoter($qid);

        if ($id) {
            $notes = getData('uid_voted');
        }
        elsif ( exists $all_aid{$aid} ) {
            $notes = getData( 'success', { aid => $aid } );
            $slashdb->createPollVoter( $qid, $aid );
            $question->{voters}++;
        }
        else {
            $notes = getData( 'reject', { aid => $aid } );
        }
    }

    my $answers = $slashdb->getPollAnswers( $qid, [ 'answer', 'votes' ] );
    my $maxvotes = $slashdb->getPollVotesMax($qid);
    my @pollitems;
    for (@$answers) {
        my ( $answer, $votes ) = @$_;
        my $imagewidth =
          $maxvotes
          ? int( 350 * $votes / $maxvotes ) + 1
          : 0;
        my $percent =
          $question->{voters}
          ? int( 100 * $votes / $question->{voters} )
          : 0;
        push @pollitems, [ $answer, $imagewidth, $votes, $percent ];
    }

    slashDisplay(
        'vote',
        {
            qid       => $qid,
            width     => '99%',
            title     => $question->{question},
            voters    => $question->{voters},
            pollitems => \@pollitems,
            notes     => $notes
        }
    );
}

#################################################################
sub deletepolls {
    my ($form) = @_;
    if ( getCurrentUser('is_admin') ) {
        my $slashdb = getCurrentDB();
        $slashdb->deletePoll( $form->{'qid'} );
    }
    listpolls(@_);
}

#################################################################
sub listpolls {
    my ($form) = @_;
    my $slashdb = getCurrentDB();
    my $min       = $form->{min} || 0;
    my $questions = $slashdb->getPollQuestionList($min);
    my $sitename  = getCurrentStatic('sitename');

    # Just me, but shouldn't title be in the template?
    # yes
    slashDisplay(
        'listpolls',
        {
            questions => $questions,
            startat   => $min + @$questions,
            admin     => getCurrentUser('seclev') >= 100,
            title     => "$sitename Polls",
            width     => '99%'
        }
    );
}



1;
